@@ -15,7 +15,6 @@ from abc import ABCMeta
 from typing_extensions import _AnnotatedAlias, Literal
 import inspect
 import warnings
-import os
 from enum import Enum
 import warnings
 from docstring_parser import parse, compose
@@ -51,6 +50,7 @@ from .mgui_ext import (
 )
 from .utils import copy_class, get_parameters, callable_to_classes
 from ._macro import GuiMacro
+from ._icon import get_icon
 
 from ..utils import (
     get_signature,
@@ -85,6 +85,7 @@ class PopUpMode(Enum):
     dock = "dock"
     dialog = "dialog"
     parentlast = "parentlast"
+    parentsub = "parentsub"
 
 
 def _msgbox_raising(e, parent):
@@ -149,42 +150,44 @@ defaults = {
     "macro-max-history": 1000,
 }
 
-_RESERVED = {
-    "__magicclass_parent__",
-    "__magicclass_children__",
-    "_close_on_run",
-    "_error_mode",
-    "_popup_mode",
-    "_my_symbol",
-    "_macro_instance",
-    "macro",
-    "annotation",
-    "enabled",
-    "find_ancestor",
-    "gui_only",
-    "height",
-    "label_changed",
-    "label",
-    "layout",
-    "labels",
-    "margins",
-    "max_height",
-    "max_width",
-    "min_height",
-    "min_width",
-    "name",
-    "options",
-    "param_kind",
-    "parent_changed",
-    "tooltip",
-    "visible",
-    "widget_type",
-    "width",
-    "wraps",
-    "_unwrap_method",
-    "_search_parent_magicclass",
-    "_iter_child_magicclasses",
-}
+_RESERVED = frozenset(
+    {
+        "__magicclass_parent__",
+        "__magicclass_children__",
+        "_close_on_run",
+        "_error_mode",
+        "_popup_mode",
+        "_my_symbol",
+        "_macro_instance",
+        "macro",
+        "annotation",
+        "enabled",
+        "find_ancestor",
+        "gui_only",
+        "height",
+        "label_changed",
+        "label",
+        "layout",
+        "labels",
+        "margins",
+        "max_height",
+        "max_width",
+        "min_height",
+        "min_width",
+        "name",
+        "options",
+        "param_kind",
+        "parent_changed",
+        "tooltip",
+        "visible",
+        "widget_type",
+        "width",
+        "wraps",
+        "_unwrap_method",
+        "_search_parent_magicclass",
+        "_iter_child_magicclasses",
+    }
+)
 
 
 def check_override(cls: type):
@@ -245,7 +248,7 @@ class MagicTemplate(metaclass=_MagicTemplateMeta):
     enabled: bool
     gui_only: bool
     height: int
-    icon_path: str
+    icon: Any
     label_changed: Signal
     label: str
     layout: str
@@ -377,7 +380,7 @@ class MagicTemplate(metaclass=_MagicTemplateMeta):
             current_self = current_self.__magicclass_parent__
             if current_self is None:
                 raise RuntimeError(
-                    f"Magic class {ancestor.__name__} not found. {ancestor.__name__} it "
+                    f"Magic class {ancestor.__name__} not found. {ancestor.__name__} "
                     f"is not an ancestor of {self.__class__.__name__}"
                 )
         return current_self
@@ -647,6 +650,7 @@ class MagicTemplate(metaclass=_MagicTemplateMeta):
                     if self._popup_mode not in (
                         PopUpMode.popup,
                         PopUpMode.dock,
+                        PopUpMode.parentsub,
                         PopUpMode.dialog,
                     ):
                         mgui.label = ""
@@ -662,10 +666,15 @@ class MagicTemplate(metaclass=_MagicTemplateMeta):
                         mgui.insert(0, title)
 
                     if self._close_on_run and not mgui._auto_call:
-                        if self._popup_mode not in (PopUpMode.dock, PopUpMode.dialog):
+                        if self._popup_mode not in (
+                            PopUpMode.dock,
+                            PopUpMode.parentsub,
+                            PopUpMode.dialog,
+                        ):
                             mgui.called.connect(mgui.hide)
-                        elif self._popup_mode == PopUpMode.dock:
-                            # If FunctioGui is docked, we should close QDockWidget.
+                        elif self._popup_mode in (PopUpMode.dock, PopUpMode.parentsub):
+                            # If FunctioGui is docked or in a subwindow, we should close
+                            # the parent QDockWidget/QMdiSubwindow.
                             mgui.called.connect(lambda: mgui.parent.hide())
 
                 if nparams == 1 and issubclass(fgui_classes[0], FileEdit):
@@ -680,10 +689,16 @@ class MagicTemplate(metaclass=_MagicTemplateMeta):
                     else:
                         return None
 
-                if self._popup_mode not in (PopUpMode.dock, PopUpMode.dialog):
+                if self._popup_mode not in (
+                    PopUpMode.dock,
+                    PopUpMode.dialog,
+                    PopUpMode.parentsub,
+                ):
                     widget.mgui.show()
                 elif self._popup_mode == PopUpMode.dock:
                     mgui.parent.show()  # show dock widget
+                elif self._popup_mode == PopUpMode.parentsub:
+                    mgui.native.parent().setVisible(True)
                 else:
                     mgui.exec_as_dialog(parent=self)
 
@@ -734,10 +749,30 @@ class BaseGui(MagicTemplate):
         self.__magicclass_parent__: BaseGui | None = None
         self.__magicclass_children__: list[MagicTemplate] = []
         self._close_on_run = close_on_run
-        self._popup_mode = popup_mode
-        self._error_mode = error_mode
+        self._popup_mode = popup_mode or PopUpMode.popup
+        self._error_mode = error_mode or ErrorMode.msgbox
         self._my_symbol = Symbol.var("ui")
-        self._icon_path = None
+        self._icon = None
+
+        self.macro.widget.__magicclass_parent__ = self
+
+    @property
+    def icon(self):
+        return self._icon
+
+    @icon.setter
+    def icon(self, val):
+        icon = get_icon(val)
+        qicon = icon.get_qicon(self)
+        self._icon = icon
+        if hasattr(self.native, "setIcon"):
+            self.native.setIcon(qicon)
+        else:
+            self.native.setWindowIcon(qicon)
+        if hasattr(self.native, "setIconSize"):
+            self.native.setIconSize(self.native.size())
+
+    icon_path = icon
 
 
 class ContainerLikeGui(BaseGui, mguiLike, MutableSequence):
@@ -747,25 +782,6 @@ class ContainerLikeGui(BaseGui, mguiLike, MutableSequence):
     changed = Signal(object)
     _list: list[AbstractAction | ContainerLikeGui]
 
-    @property
-    def icon_path(self):
-        return self._icon_path
-
-    @icon_path.setter
-    def icon_path(self, path: str):
-        path = str(path)
-        if os.path.exists(path):
-            icon = QIcon(path)
-            if hasattr(self.native, "setIcon"):
-                self.native.setIcon(icon)
-            else:
-                self.native.setWindowIcon(icon)
-            self._icon_path = path
-        else:
-            warnings.warn(
-                f"Path {path} does not exists. Could not set icon.", UserWarning
-            )
-
     def reset_choices(self, *_: Any):
         """Reset child Categorical widgets"""
         all_widgets: set[Widget] = set()
@@ -1386,6 +1402,14 @@ def _define_popup(self: BaseGui, obj, widget: PushButtonPlus | Action):
                     mgui, name=_get_widget_name(widget), area="right"
                 )
 
+    elif popup_mode == PopUpMode.parentsub:
+
+        def _prep(mgui: FunctionGui):
+            from .class_gui import find_window_ancestor
+
+            parent_self = find_window_ancestor(self)
+            parent_self._widget._mdiarea.addSubWindow(mgui.native)
+
     elif popup_mode == PopUpMode.dialog:
 
         def _prep(mgui: FunctionGui):
