@@ -20,11 +20,7 @@ from ._base import (
     value_widget_callback,
     nested_function_gui_callback,
 )
-from .utils import (
-    copy_class,
-    format_error,
-    set_context_menu,
-)
+from .utils import format_error, connect_magicclasses
 from ..widgets import (
     ButtonContainer,
     GroupBoxContainer,
@@ -53,6 +49,7 @@ _USE_OUTER_LAYOUT = (
     DraggableContainer,
     SplitterContainer,
     TabbedContainer,
+    SubWindowsContainer,
 )
 
 _MCLS_PAREMT = "__magicclass_parent__"
@@ -142,9 +139,7 @@ class ClassGuiBase(BaseGui):
                     widget = getattr(self, name, None)
 
                 if isinstance(widget, BaseGui):
-                    widget.__magicclass_parent__ = self
-                    self.__magicclass_children__.append(widget)
-                    widget._my_symbol = Symbol(name)
+                    connect_magicclasses(self, widget, name)
 
                 if isinstance(widget, MenuGui):
                     # Add menubar to container
@@ -176,7 +171,7 @@ class ClassGuiBase(BaseGui):
 
                 elif isinstance(widget, ContextMenuGui):
                     # Add context menu to container
-                    set_context_menu(widget, self)
+                    widget._set_magic_context_menu(self)
                     _hist.append((name, type(attr), "ContextMenuGui"))
 
                 elif isinstance(widget, ToolBarGui):
@@ -232,6 +227,13 @@ class ClassGuiBase(BaseGui):
                             )
                             continue
 
+                        # contextmenu
+                        contextmenu = get_additional_option(attr, "context_menu", None)
+                        if contextmenu is not None:
+                            contextmenu: ContextMenuGui
+                            contextmenu._set_magic_context_menu(widget)
+                            connect_magicclasses(self, contextmenu, contextmenu.name)
+
                     elif hasattr(widget, _MCLS_PAREMT) or hasattr(
                         widget.__class__, _MCLS_PAREMT
                     ):
@@ -334,10 +336,29 @@ class ClassGuiBase(BaseGui):
         return None
 
 
+def find_window_ancestor(widget: Widget) -> SubWindowsClassGui:
+    parent_self = widget
+    while (parent := getattr(parent_self, "__magicclass_parent__", None)) is not None:
+        parent_self = parent
+        if isinstance(parent_self, SubWindowsClassGui):
+            break
+
+    if not isinstance(parent_self, SubWindowsClassGui):
+        raise RuntimeError(
+            "Could not find GUI class that support sub-windows. Please use\n"
+            ">>> @magicclass(widget_type='subwindows')\n"
+            "to create main window widget."
+        )
+    return parent_self
+
+
 _C = TypeVar("_C", bound=ContainerWidget)
+_C2 = TypeVar("_C2")
 
 
-def make_gui(container: type[_C], no_margin: bool = True) -> type[_C | ClassGuiBase]:
+def make_gui(
+    container: type[_C], no_margin: bool = True
+) -> Callable[[_C2], type[_C | _C2 | ClassGuiBase]]:
     """
     Make a ClassGui class from a Container widget.
     Because GUI class inherits Container here, functions that need overriden must be defined
