@@ -1,8 +1,9 @@
 from __future__ import annotations
 from functools import wraps as functools_wraps
 import inspect
+import warnings
 from weakref import WeakValueDictionary
-from typing import Any, TYPE_CHECKING
+from typing import Any, TYPE_CHECKING, Callable
 
 from ._gui.class_gui import (
     ClassGuiBase,
@@ -33,7 +34,7 @@ from ._gui._base import (
 from ._gui import ContextMenuGui, MenuGui, ToolBarGui
 from ._app import get_app
 from .types import WidgetType
-from . import _register  # activate type registration things.
+from . import _register_types  # activate type registration things.
 
 if TYPE_CHECKING:
     from .stylesheets import StyleSheet
@@ -75,8 +76,9 @@ def magicclass(
     popup_mode: PopUpModeStr | PopUpMode = None,
     error_mode: ErrorModeStr | ErrorMode = None,
     widget_type: WidgetTypeStr | WidgetType = WidgetType.none,
-    icon_path: str = None,
+    icon: Any | None = None,
     stylesheet: str | StyleSheet = None,
+    icon_path: str = None,
 ):
     """
     Decorator that can convert a Python class into a widget.
@@ -112,8 +114,8 @@ def magicclass(
         Option of how to raise errors during function calls.
     widget_type : WidgetType or str, optional
         Widget type of container.
-    icon_path : str, optional
-        Path to the icon image.
+    icon : Any, optional
+        Path to the icon image or any object that can be converted into an icon.
     stylesheet : str or StyleSheet object, optional
         Set stylesheet to the widget if given.
 
@@ -127,6 +129,11 @@ def magicclass(
         close_on_run = defaults["close_on_run"]
     if error_mode is None:
         error_mode = defaults["error_mode"]
+    if icon_path is not None:
+        icon = icon_path
+        warnings.warn(
+            "`icon_path` is deprecated. Use `icon` instead.", DeprecationWarning
+        )
 
     if isinstance(widget_type, str):
         widget_type = widget_type.lower()
@@ -189,7 +196,6 @@ def magicclass(
             # prepare macro
             macrowidget = self.macro.widget.native
             macrowidget.setParent(self.native, macrowidget.windowFlags())
-            self.macro.widget.__magicclass_parent__ = self
 
             with self.macro.blocked():
                 super(oldclass, self).__init__(*args, **kwargs)
@@ -199,8 +205,8 @@ def magicclass(
             if widget_type in (WidgetType.collapsible, WidgetType.button):
                 self.text = self.name
 
-            if icon_path:
-                self.icon_path = icon_path
+            if icon:
+                self.icon = icon
             if stylesheet:
                 self.native.setStyleSheet(str(stylesheet))
             if hasattr(self, "__post_init__"):
@@ -228,28 +234,118 @@ def magicmenu(
     error_mode: str | ErrorMode = None,
     labels: bool = True,
     name: str | None = None,
+    icon: Any | None = None,
     icon_path: str = None,
 ):
-    """
-    Decorator that can convert a Python class into a menu bar.
-    """
+    """Decorator that converts a Python class into a menu bar."""
     return _call_magicmenu(**locals(), menugui_class=MenuGui)
 
 
 def magiccontext(
     class_: type = None,
     *,
+    into: Callable | None = None,
     close_on_run: bool = None,
     popup_mode: str | PopUpMode = None,
     error_mode: str | ErrorMode = None,
     labels: bool = True,
     name: str | None = None,
+    icon: Any | None = None,
     icon_path: str = None,
 ):
-    """
-    Decorator that can convert a Python class into a context menu.
-    """
-    return _call_magicmenu(**locals(), menugui_class=ContextMenuGui)
+    """Decorator that converts a Python class into a context menu."""
+
+    if popup_mode is None:
+        popup_mode = defaults["popup_mode"]
+    if close_on_run is None:
+        close_on_run = defaults["close_on_run"]
+    if error_mode is None:
+        error_mode = defaults["error_mode"]
+    if icon_path is not None:
+        icon = icon_path
+        warnings.warn(
+            "`icon_path` is deprecated. Use `icon` instead.", DeprecationWarning
+        )
+
+    if popup_mode in (
+        PopUpMode.above,
+        PopUpMode.below,
+        PopUpMode.first,
+        PopUpMode.last,
+    ):
+        raise ValueError(f"Mode {popup_mode.value} is not compatible with Menu.")
+
+    def wrapper(cls) -> type[ContextMenuGui]:
+        if not isinstance(cls, type):
+            raise TypeError(f"magicclass can only wrap classes, not {type(cls)}")
+
+        if not issubclass(cls, MagicTemplate):
+            check_override(cls)
+
+        # get class attributes first
+        doc = cls.__doc__
+        sig = inspect.signature(cls)
+        mod = cls.__module__
+        qualname = cls.__qualname__
+
+        new_attrs = convert_attributes(cls, hide=ContextMenuGui.__mro__)
+        oldclass = type(cls.__name__ + _BASE_CLASS_SUFFIX, (cls,), {})
+        newclass = type(cls.__name__, (ContextMenuGui, oldclass), new_attrs)
+
+        newclass.__signature__ = sig
+        newclass.__doc__ = doc
+        newclass.__module__ = mod
+        newclass.__qualname__ = qualname
+
+        @functools_wraps(oldclass.__init__)
+        def __init__(self: MagicTemplate, *args, **kwargs):
+            # Without "app = " Jupyter freezes after closing the window!
+            app = get_app()
+
+            gui_kwargs = dict(
+                close_on_run=close_on_run,
+                popup_mode=PopUpMode(popup_mode),
+                error_mode=ErrorMode(error_mode),
+                labels=labels,
+                name=name or cls.__name__,
+            )
+
+            # Inheriting Container's constructor is the most intuitive way.
+            if kwargs and "__init__" not in cls.__dict__:
+                gui_kwargs.update(kwargs)
+                kwargs = {}
+
+            ContextMenuGui.__init__(
+                self,
+                **gui_kwargs,
+            )
+
+            macrowidget = self.macro.widget.native
+            macrowidget.setParent(self.native, macrowidget.windowFlags())
+
+            with self.macro.blocked():
+                super(oldclass, self).__init__(*args, **kwargs)
+
+            self._convert_attributes_into_widgets()
+
+            if icon:
+                self.icon = icon
+            if hasattr(self, "__post_init__"):
+                with self.macro.blocked():
+                    self.__post_init__()
+            if into is not None:
+                from .signature import upgrade_signature
+
+                upgrade_signature(into, additional_options={"context_menu": self})
+
+        newclass.__init__ = __init__
+
+        # Users may want to override repr
+        newclass.__repr__ = oldclass.__repr__
+
+        return newclass
+
+    return wrapper if class_ is None else wrapper(class_)
 
 
 def magictoolbar(
@@ -260,93 +356,13 @@ def magictoolbar(
     error_mode: str | ErrorMode = None,
     labels: bool = True,
     name: str | None = None,
+    icon: Any | None = None,
     icon_path: str = None,
 ):
-    """
-    Decorator that can convert a Python class into a menu bar.
-    """
+    """Decorator that converts a Python class into a menu bar."""
     return _call_magicmenu(**locals(), menugui_class=ToolBarGui)
 
 
-class MagicClassFactory:
-    """
-    Factory class that can make any magic-class.
-    """
-
-    def __init__(
-        self,
-        name: str,
-        layout: str = "vertical",
-        labels: bool = True,
-        close_on_run: bool = None,
-        popup_mode: str | PopUpMode = None,
-        error_mode: str | ErrorMode = None,
-        widget_type: str | WidgetType = WidgetType.none,
-        icon_path: str = None,
-        attrs: dict[str, Any] | None = None,
-    ):
-        self.name = name
-        self.layout = layout
-        self.labels = labels
-        self.close_on_run = close_on_run
-        self.popup_mode = popup_mode
-        self.error_mode = error_mode
-        self.widget_type = widget_type
-        self.icon_path = icon_path
-        self.attrs = attrs
-
-    def as_magicclass(self) -> ClassGuiBase:
-        _cls = type(self.name, (), self.attrs)
-        cls = magicclass(
-            _cls,
-            layout=self.layout,
-            labels=self.labels,
-            close_on_run=self.close_on_run,
-            name=self.name,
-            popup_mode=self.popup_mode,
-            error_mode=self.error_mode,
-            widget_type=self.widget_type,
-            icon_path=self.icon_path,
-        )
-        return cls
-
-    def as_magictoolbar(self) -> ToolBarGui:
-        _cls = type(self.name, (), self.attrs)
-        cls = magictoolbar(
-            _cls,
-            close_on_run=self.close_on_run,
-            popup_mode=self.popup_mode,
-            error_mode=self.error_mode,
-            labels=self.labels,
-            icon_path=self.icon_path,
-        )
-        return cls
-
-    def as_magicmenu(self) -> MenuGui:
-        _cls = type(self.name, (), self.attrs)
-        cls = magicmenu(
-            _cls,
-            close_on_run=self.close_on_run,
-            popup_mode=self.popup_mode,
-            error_mode=self.error_mode,
-            labels=self.labels,
-            icon_path=self.icon_path,
-        )
-        return cls
-
-    def as_magiccontext(self) -> ContextMenuGui:
-        _cls = type(self.name, (), self.attrs)
-        cls = magiccontext(
-            _cls,
-            close_on_run=self.close_on_run,
-            popup_mode=self.popup_mode,
-            error_mode=self.error_mode,
-            labels=self.labels,
-            icon_path=self.icon_path,
-        )
-        return cls
-
-
 def _call_magicmenu(
     class_: type = None,
     close_on_run: bool = True,
@@ -354,6 +370,7 @@ def _call_magicmenu(
     error_mode: str | ErrorMode = None,
     labels: bool = True,
     name: str = None,
+    icon: Any | None = None,
     icon_path: str = None,
     menugui_class: type[MenuGuiBase] = None,
 ):
@@ -365,7 +382,7 @@ def _call_magicmenu(
     close_on_run : bool, default is True
         If True, magicgui created by every method will be deleted after the method is completed without
         exceptions, i.e. magicgui is more like a dialog.
-    popup : bool, default is True
+    popup_mode : bool, default is True
         If True, magicgui created by every method will be poped up, else they will be appended as a
         part of the main widget.
 
@@ -380,6 +397,11 @@ def _call_magicmenu(
         close_on_run = defaults["close_on_run"]
     if error_mode is None:
         error_mode = defaults["error_mode"]
+    if icon_path is not None:
+        icon = icon_path
+        warnings.warn(
+            "`icon_path` is deprecated. Use `icon` instead.", DeprecationWarning
+        )
 
     if popup_mode in (
         PopUpMode.above,
@@ -436,15 +458,14 @@ def _call_magicmenu(
 
             macrowidget = self.macro.widget.native
             macrowidget.setParent(self.native, macrowidget.windowFlags())
-            self.macro.widget.__magicclass_parent__ = self
 
             with self.macro.blocked():
                 super(oldclass, self).__init__(*args, **kwargs)
 
             self._convert_attributes_into_widgets()
 
-            if icon_path:
-                self.icon_path = icon_path
+            if icon:
+                self.icon = icon
             if hasattr(self, "__post_init__"):
                 with self.macro.blocked():
                     self.__post_init__()
@@ -526,17 +547,17 @@ def get_function_gui(ui: MagicTemplate, name: str) -> FunctionGuiPlus:
     return mgui
 
 
-def redo(ui: MagicTemplate, index: int = -1) -> None:
+def repeat(ui: MagicTemplate, index: int = -1) -> None:
     """
-    Redo operation on GUI using recorded macro.
+    Repeat last operation on GUI using recorded macro.
 
     Parameters
     ----------
     ui : MagicTemplate
         Target magic-class widget.
     index : int, default is -1
-        Which execution will be redone. Any object that support list slicing can be used.
-        By default the last operation will be redone.
+        Which execution will be repeated. Any object that support list slicing can be used.
+        By default the last operation will be repeated.
     """
     line = ui.macro[index]
     try:
